@@ -1,4 +1,6 @@
 ;;; u-vm-color.el --- Font-lock support for VM.
+;;
+;; This file is an add-on for VM
 
 ;;  Copyright (C) 2001-2007 by Ulf Jasper
 
@@ -10,6 +12,14 @@
 ;;  Time-stamp: "23. Februar 2008, 21:28:20 (ulf)"
 ;;  CVS-Version: $Id: u-vm-color.el,v 2.19 2008-02-23 20:28:57 ulf Exp $
 
+;;; Code
+
+(provide 'u-vm-color)
+
+(eval-when-compile
+  (require 'vm-misc)
+  (require 'vm-folder))
+
 (defconst u-vm-color-version "2.10" "Version number of u-vm-color.")
 
 ;;  This program is free software; you can redistribute it and/or modify it
@@ -43,16 +53,20 @@
 ;;  the following in your VM startup file (~/.emacs or ~/.vm)
 
 ;;  (require 'u-vm-color)
-;;  (add-hook 'vm-summary-mode-hook 'u-vm-color-summary-mode)
 ;;  (add-hook 'vm-select-message-hook 'u-vm-color-fontify-buffer)
 
 ;;  It may be necessary to add the following, which probably comes from
 ;;  a bug in my code...
+;;  (defadvice vm-show-current-message (after u-vm-color activate)
+;;    (u-vm-color-fontify-buffer-even-more))
+;;  You might also add this advice, which causes some slow down:
 ;;  (defadvice vm-decode-mime-message (after u-vm-color activate)
 ;;    (u-vm-color-fontify-buffer-even-more))
+;;
 
-;;  If you are using auto-fill, ie when the variable
-;;  `vm-fill-paragraphs-containing-long-lines' is not nil, you should
+;;  If you are using auto-fill, ie when the variables
+;;  `vm-word-wrap-paragraphs' or
+;;  `vm-fill-paragraphs-containing-long-lines' is not nil, you should 
 ;;  also add this:
 ;;  (defadvice vm-fill-paragraphs-containing-long-lines
 ;;              (after u-vm-color activate)
@@ -80,6 +94,9 @@
 ;; ======================================================================
 ;;; History:
 
+;;        (2011-02-17)
+;;	  Removed instructions for fontifying summary buffers because
+;;        vm-summary-faces is now built into VM.  Uday S. Reddy
 ;;  2.10: (2008-02-23)
 ;;        Bugfixes -- thanks to Martin Schwenke
 ;;  2.9:  (2007-12-19)
@@ -148,7 +165,7 @@
 
 (defgroup u-vm-color nil
   "Font-lock support for vm."
-  :group 'vm)
+  :group 'vm-ext)
 
 (defcustom u-vm-color-use-gnus-faces nil
   "Use corresponding face definitions from Gnus."
@@ -561,7 +578,7 @@ subexpressions."
 ;;;###autoload
 (defun u-vm-color-summary-mode (&optional arg)
   "Configure `font-lock-keywords' and add some hooks for vm-buffers.
-Optional argument ARG is not used!"
+ (Optional argument ARG is not used.)"
   (interactive "P")
   (setq u-vm-color-summary-mode
 	(not (or (and (null arg) u-vm-color-summary-mode)
@@ -608,6 +625,8 @@ Optional argument ARG is not used!"
               u-vm-color-summary-keywords)
 	 (set (make-local-variable 'font-lock-keywords-only) t)
  	 (font-lock-mode 1))))
+(make-obsolete 'u-vm-color-summary-mode 
+	       'vm-summary-enable-faces "8.2.0")
 
 (defun u-vm-color-fontify-regexp (start end regexp how)
   "Search the buffer for an expression and fontify it.
@@ -670,7 +689,7 @@ Search is restricted to the region between START and END."
   ;;(message "u-vm-color-fontify-buffer")
   (let ((continued-header-contents "\\(.*\\(\n[ \t]+.*\\)*\\)")
 	(pmin (point-min))
-	(buffer-has-been-modified-before (buffer-modified-p))
+	(buffer-modified (buffer-modified-p))
 	(header-end (or
 		     (save-excursion
 		       (goto-char (point-min))
@@ -726,7 +745,7 @@ Search is restricted to the region between START and END."
 			       (concat "^Content preview:"
 				       "\\([^\n]*\n\\( +[^\n]*\n\\)*\\)")
 			       '((1 u-vm-color-spamassassin-face)))
-    (set-buffer-modified-p buffer-has-been-modified-before)))
+    (vm-restore-buffer-modified-p  buffer-modified (current-buffer))))
 
 ;;;###autoload
 (defun u-vm-color-fontify-buffer-even-more ()
@@ -736,5 +755,4 @@ Search is restricted to the region between START and END."
     ;;(message "u-vm-color-fontify-even-more: %d %d" (point-min) (point-max))
     (u-vm-color-fontify-buffer)))
 
-(provide 'u-vm-color)
 ;;; u-vm-color.el ends here
