@@ -1,5 +1,7 @@
 ;;; vm-motion.el --- Commands to move around in a VM folder
 ;;
+;; This file is part of VM
+;;
 ;; Copyright (C) 1989-1997 Kyle E. Jones
 ;; Copyright (C) 2003-2006 Robert Widhopf-Fenk
 ;;
@@ -18,7 +20,24 @@
 ;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 ;;; Code:
+
+(provide 'vm-motion)
+
+(eval-when-compile
+  (require 'vm-misc)
+  (require 'vm-minibuf)
+  (require 'vm-folder)
+  (require 'vm-summary)
+  (require 'vm-thread)
+  (require 'vm-window)
+  (require 'vm-page)
+  )
+
+(declare-function vm-so-sortable-subject "vm-sort" (message))
+
 (defun vm-record-and-change-message-pointer (old new)
+  "Change the `vm-message-pointer' of the folder from OLD to NEW, both
+of which must be pointers into the `vm-message-list'."
   (intern (buffer-name) vm-buffers-needing-display-update)
   (vm-garbage-collect-message)
   (setq vm-last-message-pointer old
@@ -46,56 +65,66 @@ given."
 	     (vm-read-number "Go to message: "))))))
   (if (null n)
       ()				; nil means work has been done already
-    (vm-select-folder-buffer)
-    (vm-check-for-killed-summary)
-    (vm-error-if-folder-empty)
+    (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
     (vm-display nil nil '(vm-goto-message) '(vm-goto-message))
     (let ((cons (nthcdr (1- n) vm-message-list)))
       (if (null cons)
 	  (error "No such message."))
       (if (eq vm-message-pointer cons)
-	  (vm-preview-current-message)
+	  (vm-present-current-message)
 	(vm-record-and-change-message-pointer vm-message-pointer cons)
-	(vm-preview-current-message)))))
+	(vm-present-current-message)
+	;;(vm-warn 0 0 "start of message you want is: %s"
+	;; (vm-su-start-of (car vm-message-pointer)))
+	(if (and (vm-summary-operation-p)
+		 vm-summary-show-threads
+		 (get-text-property 
+		  (+ (vm-su-start-of (car vm-message-pointer)) 2)
+		  'invisible vm-summary-buffer))
+	    (vm-expand-thread (vm-thread-root (car vm-message-pointer))))
+	))))
 
 ;;;###autoload
 (defun vm-goto-message-last-seen ()
   "Go to the message last previewed."
   (interactive)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
-  (vm-error-if-folder-empty)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-display nil nil '(vm-goto-message-last-seen)
 	      '(vm-goto-message-last-seen))
   (if vm-last-message-pointer
       (progn
 	(vm-record-and-change-message-pointer vm-message-pointer
 					      vm-last-message-pointer)
-	(vm-preview-current-message))))
+	(vm-present-current-message))))
+(defalias 'vm-goto-last-message-seen 'vm-goto-message-last-seen)
 
 ;;;###autoload
 (defun vm-goto-parent-message ()
   "Go to the parent of the current message."
   (interactive)
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
-  (vm-error-if-folder-empty)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-build-threads-if-unbuilt)
   (vm-display nil nil '(vm-goto-parent-message)
 	      '(vm-goto-parent-message))
-  (let ((list (vm-th-thread-list (car vm-message-pointer)))
-	message)
-    (if (null (cdr list))
-	(message "Message has no parent.")
-      (while (cdr (cdr list))
-	(setq list (cdr list)))
-      (setq message (car (get (car list) 'messages)))
-      (if (null message)
-	  (message "Parent message is not in this folder.")
-	(vm-record-and-change-message-pointer vm-message-pointer
-					      (memq message vm-message-list))
-	(vm-preview-current-message)))))
+  (let ((lineage (cdr (reverse (vm-thread-list (car vm-message-pointer)))))
+	(message nil))
+    (cond ((null lineage)
+	   (vm-inform 5 "Message has no parent listed."))
+	  ((vm-th-messages-of (car lineage))
+	   (setq message (car lineage)))
+	  ((y-or-n-p (concat "Parent message is not in this folder. "
+			     "Go to the next ancestor? "))
+	   (while (and lineage (null (vm-th-messages-of (car lineage))))
+	     (setq lineage (cdr lineage)))
+	   (if (null lineage)
+	       (vm-inform 5 "Message has no ancestors in this folder")
+	     (setq message (car lineage)))))
+    (when message
+      (setq message (car (vm-th-messages-of (car lineage))))
+      (vm-record-and-change-message-pointer vm-message-pointer
+					    (vm-message-position message))
+      (vm-present-current-message))))
 
 (defun vm-check-count (count)
   (if (>= count 0)
@@ -106,6 +135,8 @@ given."
 	(signal 'beginning-of-folder nil))))
 
 (defun vm-move-message-pointer (direction)
+  "Move vm-message-pointer along DIRECTION by one position.  DIRECTION
+is one of 'forward and 'backward.                     USR, 2011-01-18"
   (let ((mp vm-message-pointer))
     (if (eq direction 'forward)
 	(progn
@@ -122,23 +153,36 @@ given."
     (setq vm-message-pointer mp)))
 
 (defun vm-should-skip-message (mp &optional skip-dogmatically)
-  (if skip-dogmatically
-      (or (and vm-skip-deleted-messages
-	       (vm-deleted-flag (car mp)))
-	  (and vm-skip-read-messages
-	       (or (vm-deleted-flag (car mp))
-		   (not (or (vm-new-flag (car mp))
-			    (vm-unread-flag (car mp))))))
-	  (and (eq last-command 'vm-next-command-uses-marks)
-	       (null (vm-mark-of (car mp)))))
-    (or (and (eq vm-skip-deleted-messages t)
-	     (vm-deleted-flag (car mp)))
-	(and (eq vm-skip-read-messages t)
-	     (or (vm-deleted-flag (car mp))
-		 (not (or (vm-new-flag (car mp))
-			  (vm-unread-flag (car mp))))))
-	(and (eq last-command 'vm-next-command-uses-marks)
-	     (null (vm-mark-of (car mp)))))))
+  "Checks various preference settings and message attributes to
+determine whether the current message should be skipped during
+movement.  The first argument MP is a pointer into the message-list.
+The optional argument SKIP-DOGMATICALLY asks it to follow a strong
+interpretation of the preferences.                  USR, 2011-01-18"
+  (or (and (if skip-dogmatically 
+	       vm-skip-deleted-messages
+	     (eq vm-skip-deleted-messages t))
+	   (vm-deleted-flag (car mp)))
+      (vm-should-skip-hidden-message mp)
+      (and (if skip-dogmatically 
+	       vm-skip-read-messages
+	     (eq vm-skip-read-messages t))
+	   (or (vm-deleted-flag (car mp))
+	       (not (or (vm-new-flag (car mp))
+			(vm-unread-flag (car mp))))))
+      (and (eq last-command 'vm-next-command-uses-marks)
+	   (null (vm-mark-of (car mp))))))
+
+(defun vm-should-skip-hidden-message (mp)
+  "Checks if the current message in MP should be skipped as a hidden
+message in the summary buffer."
+  (and vm-summary-buffer
+       (with-current-buffer vm-summary-buffer
+	 (and vm-skip-collapsed-sub-threads
+	      vm-summary-enable-thread-folding
+	      vm-summary-show-threads
+	      (> (vm-thread-indentation (car mp)) 0)
+	      (vm-collapsed-root-p (vm-thread-root (car mp)))
+	      (get-text-property (vm-su-start-of (car mp)) 'invisible)))))
 
 ;;;###autoload
 (defun vm-next-message (&optional count retry signal-errors)
@@ -148,7 +192,7 @@ messages.  A negative COUNT means go backward.  If the absolute
 value of COUNT is greater than 1, then the values of the variables
 vm-skip-deleted-messages and vm-skip-read-messages are ignored.
 
-When invoked on marked messages (via vm-next-command-uses-marks)
+When invoked on marked messages (via `vm-next-command-uses-marks')
 this command 'sees' marked messages as it moves."
   ;; second arg RETRY non-nil means retry a failed move, giving
   ;; not nil-or-t values of the vm-skip variables a chance to
@@ -162,11 +206,11 @@ this command 'sees' marked messages as it moves."
   ;;
   ;; Note that interactively all args are 1, so error signaling
   ;; and retries apply to all interactive moves.
-  (interactive "p\np\np")
-  (if (interactive-p)
+  (interactive "p\np\np")  
+  ;;(vm-inform 8 "running vm next message")
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate (if signal-errors 1 0) (vm-interactive-p))
   ;; include other commands that call vm-next-message so that the
   ;; correct window configuration is applied for these particular
   ;; non-interactive calls.
@@ -175,7 +219,6 @@ this command 'sees' marked messages as it moves."
 			vm-undelete-message
 			vm-scroll-forward)
 	      (list this-command))
-  (and signal-errors (vm-error-if-folder-empty))
   (or count (setq count 1))
   (let ((oldmp vm-message-pointer)
 	(use-marks (eq last-command 'vm-next-command-uses-marks))
@@ -200,12 +243,13 @@ this command 'sees' marked messages as it moves."
 			(setq count 1)
 		      ;; reset for next pass
 		      (setq oldmp vm-message-pointer))))
-	      (vm-decrement count)))
+	      (if (not (vm-should-skip-hidden-message vm-message-pointer))
+		  (vm-decrement count))))
 	(beginning-of-folder (setq error 'beginning-of-folder))
 	(end-of-folder (setq error 'end-of-folder))))
      (t
       (condition-case ()
-	  (progn
+	  (progn	    
 	    (vm-move-message-pointer direction)
 	    (while (and (not (eq oldmp vm-message-pointer))
 			(vm-should-skip-message vm-message-pointer t))
@@ -240,6 +284,13 @@ this command 'sees' marked messages as it moves."
 	(end-of-folder
 	 ;; we bumped into the end of the folder without finding
 	 ;; a suitable stopping point; retry the move if we're allowed.
+	 (when (and (vm-summary-operation-p)
+		    (get-text-property 
+		     (vm-su-start-of (car vm-message-pointer))
+		     'invisible vm-summary-buffer))
+	   (setq error 'end-of-folder)
+	   (setq retry nil))
+
 	 (setq vm-message-pointer oldmp)
 	 ;; if the retry fails, we make sure the message pointer
 	 ;; is restored to its old value.
@@ -255,11 +306,10 @@ this command 'sees' marked messages as it moves."
 		      (setq error 'end-of-folder)
 		      oldmp )))
 	   (setq error 'end-of-folder))))))
-    (if (not (eq vm-message-pointer oldmp))
-	(progn
-	  (vm-record-and-change-message-pointer oldmp vm-message-pointer)
-	  (vm-preview-current-message)))
-    (and error signal-errors
+    (unless (eq vm-message-pointer oldmp)
+      (vm-record-and-change-message-pointer oldmp vm-message-pointer)
+      (vm-present-current-message))
+    (when (and error signal-errors)
 	 (signal error nil))))
 
 ;;;###autoload
@@ -271,9 +321,9 @@ variables vm-skip-deleted-messages and vm-skip-read-messages are
 ignored."
   (interactive "p\np\np")
   (or count (setq count 1))
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
+  (vm-select-folder-buffer-and-validate 1 t)
   (vm-display nil nil '(vm-previous-message) '(vm-previous-message))
   (vm-next-message (- count) retry signal-errors))
 
@@ -281,14 +331,28 @@ ignored."
 (defun vm-next-message-no-skip (&optional count)
   "Like vm-next-message but will not skip deleted or read messages."
   (interactive "p")
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-display nil nil '(vm-next-message-no-skip)
 	      '(vm-next-message-no-skip))
+  (when (and vm-summary-enable-thread-folding
+	     vm-summary-show-threads
+	     vm-summary-thread-folding-on-motion)    
+    (with-current-buffer vm-summary-buffer
+      (let ((msg (vm-summary-message-at-point))
+	    (root (vm-thread-root (vm-summary-message-at-point))))
+	;; if last message collapse (and do not move)
+	(if (= (string-to-number (vm-number-of msg))
+	       (+ (string-to-number (vm-number-of root)) 
+		  (- (vm-thread-count root) 1)))
+	    (vm-collapse-thread t)))))
   (let ((vm-skip-deleted-messages nil)
-	(vm-skip-read-messages nil))
+	(vm-skip-read-messages nil)
+	(vm-skip-collapsed-sub-threads 
+	 (not vm-summary-thread-folding-on-motion)))
     (vm-next-message count nil t)))
+
 ;; backward compatibility
 (fset 'vm-Next-message 'vm-next-message-no-skip)
 
@@ -296,14 +360,26 @@ ignored."
 (defun vm-previous-message-no-skip (&optional count)
   "Like vm-previous-message but will not skip deleted or read messages."
   (interactive "p")
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-display nil nil '(vm-previous-message-no-skip)
 	      '(vm-previous-message-no-skip))
+  (when (and vm-summary-enable-thread-folding
+	     vm-summary-show-threads
+	     vm-summary-thread-folding-on-motion)    
+    (with-current-buffer vm-summary-buffer
+      (let ((msg (vm-summary-message-at-point))
+	    (root (vm-thread-root (vm-summary-message-at-point))))
+	;; if root message collapse (moving up)
+	(if (eq msg root)
+	    (vm-collapse-thread t)))))
   (let ((vm-skip-deleted-messages nil)
-	(vm-skip-read-messages nil))
+	(vm-skip-read-messages nil)
+	(vm-skip-collapsed-sub-threads 
+	 (not vm-summary-thread-folding-on-motion)))
     (vm-previous-message count)))
+
 ;; backward compatibility
 (fset 'vm-Previous-message 'vm-previous-message-no-skip)
 
@@ -311,10 +387,9 @@ ignored."
 (defun vm-next-unread-message ()
   "Move forward to the nearest new or unread message, if there is one."
   (interactive)
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 0 (vm-interactive-p))
   (vm-display nil nil '(vm-next-unread-message) '(vm-next-unread-message))
   (condition-case ()
       (let ((vm-skip-read-messages t)
@@ -322,16 +397,15 @@ ignored."
 	(vm-next-message 1 nil t)
 	;; in case vm-circular-folders is non-nil
 	(and (eq vm-message-pointer oldmp) (signal 'end-of-folder nil)))
-    (end-of-folder (message "No next unread message"))))
+    (end-of-folder (vm-inform 5 "No next unread message"))))
 
 ;;;###autoload
 (defun vm-previous-unread-message ()
   "Move backward to the nearest new or unread message, if there is one."
   (interactive)
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 0 (vm-interactive-p))
   (vm-display nil nil '(vm-previous-unread-message)
 	      '(vm-previous-unread-message))
   (condition-case ()
@@ -340,7 +414,7 @@ ignored."
 	(vm-previous-message)
 	;; in case vm-circular-folders is non-nil
 	(and (eq vm-message-pointer oldmp) (signal 'beginning-of-folder nil)))
-    (beginning-of-folder (message "No previous unread message"))))
+    (beginning-of-folder (vm-inform 5 "No previous unread message"))))
 
 ;;;###autoload
 (defun vm-next-message-same-subject ()
@@ -348,10 +422,9 @@ ignored."
 vm-subject-ignored-prefix and vm-subject-ignored-suffix will apply
 to the subject comparisons."
   (interactive)
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 0 (vm-interactive-p))
   (vm-display nil nil '(vm-next-message-same-subject)
 	      '(vm-next-message-same-subject))
   (let ((oldmp vm-message-pointer)
@@ -367,10 +440,10 @@ to the subject comparisons."
 		       (vm-so-sortable-subject (car vm-message-pointer)))
 		(setq done t)))
 	  (vm-record-and-change-message-pointer oldmp vm-message-pointer)
-	  (vm-preview-current-message))
+	  (vm-present-current-message))
       (end-of-folder
        (setq vm-message-pointer oldmp)
-       (message "No next message with the same subject")))))
+       (vm-inform 5 "No next message with the same subject")))))
 
 ;;;###autoload
 (defun vm-previous-message-same-subject ()
@@ -378,10 +451,9 @@ to the subject comparisons."
 vm-subject-ignored-prefix and vm-subject-ignored-suffix will apply
 to the subject comparisons."
   (interactive)
-  (if (interactive-p)
+  (if (vm-interactive-p)
       (vm-follow-summary-cursor))
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 0 (vm-interactive-p))
   (vm-display nil nil '(vm-previous-message-same-subject)
 	      '(vm-previous-message-same-subject))
   (let ((oldmp vm-message-pointer)
@@ -397,10 +469,10 @@ to the subject comparisons."
 		       (vm-so-sortable-subject (car vm-message-pointer)))
 		(setq done t)))
 	  (vm-record-and-change-message-pointer oldmp vm-message-pointer)
-	  (vm-preview-current-message))
+	  (vm-present-current-message))
       (beginning-of-folder
        (setq vm-message-pointer oldmp)
-       (message "No previous message with the same subject")))))
+       (vm-inform 5 "No previous message with the same subject")))))
 
 (defun vm-find-first-unread-message (new)
   (let (mp unread-mp)
@@ -418,6 +490,10 @@ to the subject comparisons."
     unread-mp ))
 
 (defun vm-thoughtfully-select-message ()
+  "Select a message in the current folder for the cursor position,
+which should be the first new message, if there is any, the first
+unread message, if there is any, or the position the cursor was at
+the last time the folder was visited.  USR, 2010-03-08"
   (let ((new (and vm-jump-to-new-messages (vm-find-first-unread-message t)))
 	(unread (and vm-jump-to-unread-messages
 		     (vm-find-first-unread-message nil)))
@@ -435,7 +511,9 @@ to the subject comparisons."
 (defun vm-follow-summary-cursor ()
   "Select the message under the cursor in the summary window before
 executing commands that operate on the current message.  This occurs
-only when the summary buffer window is the selected window."
+only when the summary buffer window is the selected window.  
+
+If a new message is selected then return t, otherwise nil. USR, 2010-03-08" 
   (and vm-follow-summary-cursor (eq major-mode 'vm-summary-mode)
        (let ((point (point))
 	     message-pointer message-list mp)
@@ -450,33 +528,51 @@ only when the summary buffer window is the selected window."
 	       ;; the position at eob belongs to the last message
 	       ((and (eobp) (= (vm-su-end-of (car message-pointer)) point))
 		nil )
-	       ;; make the position at eob belong to the last message
 	       ((eobp)
-		(setq mp (vm-last message-pointer))
 		(save-excursion
-		  (set-buffer vm-mail-buffer)
-		  (vm-record-and-change-message-pointer vm-message-pointer mp)
-		  (vm-preview-current-message)
-		  ;; return non-nil so the caller will know that
-		  ;; a new message was selected.
-		  t ))
+		  (while (get-text-property (- (point) 3) 'invisible)
+		    (goto-char 
+		     (- (vm-su-start-of (get-text-property 
+					 (- (point) 3) 'vm-message))
+			3)))
+		  (vm-goto-message 
+		   (string-to-number 
+		    (vm-number-of 
+		     (get-text-property (- (point) 3) 'vm-message)))))
+		t)
+	       ;; make the position at eob belong to the last message
+	       ;; ((eobp)
+	       ;; 	(while (get-text-property (point) 'invisible)
+	       ;; 	  (goto-char (1- (point)))
+	       ;; 	  setq mp 
+	       ;; 	  ;;(setq mp (vm-last message-pointer))
+	       ;; 	(save-excursion
+	       ;; 	  (set-buffer vm-mail-buffer)
+	       ;; 	  (vm-record-and-change-message-pointer 
+	       ;;		vm-message-pointer mp)
+	       ;; 	  (vm-present-current-message)
+	       ;; 	  ;; return non-nil so the caller will know that
+	       ;; 	  ;; a new message was selected.
+	       ;; 	  t ))
 	       (t
 		(if (< point (vm-su-start-of (car message-pointer)))
 		    (setq mp message-list)
 		  (setq mp (cdr message-pointer) message-pointer nil))
-		(while (and (not (eq mp message-pointer))
-			    (>= point (vm-su-end-of (car mp))))
+		(while (or (and (not (eq mp message-pointer))
+				(>= point (vm-su-end-of (car mp))))
+			   (get-text-property 
+			    (+ (vm-su-start-of (car mp)) 3) 'invisible))
 		  (setq mp (cdr mp)))
 		(if (not (eq mp message-pointer))
 		    (save-excursion
 		      (set-buffer vm-mail-buffer)
 		      (vm-record-and-change-message-pointer
 		       vm-message-pointer mp)
-		      (vm-preview-current-message)
+		      ;; preview disabled to avoid message
+		      ;; loading. USR, 2010-09-30
+		      ;; (vm-present-current-message)
 		      ;; return non-nil so the caller will know that
 		      ;; a new message was selected.
 		      t )))))))
 
-(provide 'vm-motion)
-
 ;;; vm-motion.el ends here
