@@ -1,13 +1,14 @@
 ;;; vm-pcrisis.el --- wide-ranging auto-setup for personalities in VM
 ;;
+;; This file is an add-on for VM
+;;
 ;; Copyright (C) 1999 Rob Hodges,
 ;;               2006 Robert Widhopf, Robert P. Goldman
+;;		 2011 Uday S. Reddy
 ;;
 ;; Package: Personality Crisis for VM
 ;; Author: Rob Hodges
 ;;
-;; Maintainer: Robert Widhopf-Fenk <hack@robf.de>
-;; X-URL:       http://www.robf.de/Hacking/elisp
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -28,17 +29,22 @@
 ;; DOCUMENTATION:
 ;; -------------
 ;;
-;; Documentation is now in Texinfo and HTML formats.  You should have
-;; downloaded one or the other along with this package at the URL
-;; above.
+;; Documentation is now in Texinfo format, included
+;; in the standard VM distribution.
 
 ;;; Code:
 
+(provide 'vm-pcrisis)
+
+(eval-and-compile
+  (require 'timezone)
+  (require 'vm-misc)
+  (require 'vm-minibuf)
+  (require 'vm-folder)
+  (require 'vm-summary)
+  (require 'vm-motion)
+  (require 'vm-reply))
 (eval-when-compile
-  (require 'vm-version)
-  (require 'vm-message)
-  (require 'vm-macro)
-  (require 'vm-reply)
   ;; get the macros we need.
   (require 'cl)
   (require 'advice)
@@ -50,7 +56,16 @@
     (error
      (message "%S" e)
      (message "Could not load bbdb.el.  Related functions may not work correctly!")
-     (sit-for 5))))
+     ;; (vm-sit-for 5)
+     )))
+
+(declare-function set-extent-face "vm-xemacs" (extent face))
+(declare-function timezone-absolute-from-gregorian "ext:timezone" 
+		  (month day year))
+(declare-function bbdb-buffer "ext:bbdb" ())
+(declare-function vm-imap-account-name-for-spec "vm-imap" (maildrop-spec))
+(declare-function vm-pop-find-name-for-spec "vm-pop" (maildrop-spec))
+
 
 ;; Dummy declarations for variables that are defined in bbdb
 
@@ -66,7 +81,7 @@
 
 (defgroup vmpc nil
   "Manage personalities and more in VM."
-  :group  'vm)
+  :group  'vm-ext)
 
 (defcustom vmpc-conditions ()
   "*List of conditions which will be checked by pcrisis."
@@ -243,11 +258,13 @@ It will ensure that pcrisis correctly handles the signature .")
   "Setup pcrisis with the given IDENTITIES."
   (setq vmpc-conditions    '(("always true" t))
         vmpc-actions-alist '(("always true" "prompt for a profile"))
-        vmpc-actions       '(("prompt for a profile" (vmpc-prompt-for-profile t t))))
+        vmpc-actions       '(("prompt for a profile" 
+			      (vmpc-prompt-for-profile t t))))
   (setq vmpc-actions
         (append (mapcar
-                 (lambda (i)
-                   (list i (list 'vmpc-substitute-header "From" i)))
+                 (lambda (identity)
+		   `(,identity
+		     (vmpc-substitute-header "From" ,identity)))
                  identities)
                 vmpc-actions)))
 
@@ -256,8 +273,9 @@ It will ensure that pcrisis correctly handles the signature .")
 If point is not in a header field, returns nil."
   (save-excursion
     (unless (save-excursion
-	      (re-search-backward (regexp-quote mail-header-separator)
-				  (point-min) t))
+	      (re-search-backward 
+               (concat "^\\(" (regexp-quote mail-header-separator) "\\)$")
+	       (point-min) t))
       (re-search-backward "^\\([^ \t\n:]+\\):")
       (match-string 1))))
 
@@ -336,8 +354,8 @@ of the 'start-open and 'end-closed properties of the extent
 respectively.
 This is the XEmacs version of `vmpc-set-overlay-insertion-types'."
   ;; pretty simple huh?
-  (set-extent-property extent 'start-open start)
-  (set-extent-property extent 'end-closed end))
+  (vm-set-extent-property extent 'start-open start)
+  (vm-set-extent-property extent 'end-closed end))
 
 
 (defun vmpc-set-exerlay-insertion-types (exerlay start end)
@@ -354,28 +372,28 @@ start and end of the overlay/extent."
 (defun vmpc-exerlay-start (exerlay)
   "Return buffer position of the start of EXERLAY."
   (if vm-xemacs-p
-      (extent-start-position exerlay)
+      (vm-extent-start-position exerlay)
     (overlay-start exerlay)))
 
 
 (defun vmpc-exerlay-end (exerlay)
   "Return buffer position of the end of EXERLAY."
   (if vm-xemacs-p
-      (extent-end-position exerlay)
+      (vm-extent-end-position exerlay)
     (overlay-end exerlay)))
 
 
 (defun vmpc-move-exerlay (exerlay new-start new-end)
   "Change EXERLAY to cover region from NEW-START to NEW-END."
   (if vm-xemacs-p
-      (set-extent-endpoints exerlay new-start new-end (current-buffer))
+      (vm-set-extent-endpoints exerlay new-start new-end (current-buffer))
     (move-overlay exerlay new-start new-end (current-buffer))))
 
 
 (defun vmpc-set-exerlay-detachable-property (exerlay newval)
   "Set the 'detachable or 'evaporate property for EXERLAY to NEWVAL."
   (if vm-xemacs-p
-      (set-extent-property exerlay 'detachable newval)
+      (vm-set-extent-property exerlay 'detachable newval)
     (overlay-put exerlay 'evaporate newval)))
 
 
@@ -384,7 +402,7 @@ start and end of the overlay/extent."
   (if vm-xemacs-p
       (progn
 	(require 'atomic-extents)
-	(set-extent-property exerlay 'atomic newval))
+	(vm-set-extent-property exerlay 'atomic newval))
     (overlay-put exerlay 'intangible newval)))
 
 
@@ -398,15 +416,13 @@ start and end of the overlay/extent."
 (defun vmpc-forcefully-detach-exerlay (exerlay)
   "Leave EXERLAY in memory but detaches it from the buffer."
   (if vm-xemacs-p
-      (detach-extent exerlay)
+      (vm-detach-extent exerlay)
     (delete-overlay exerlay)))
 
 
 (defun vmpc-make-exerlay (startpos endpos)
   "Create a new exerlay spanning from STARTPOS to ENDPOS."
-  (if vm-xemacs-p
-      (make-extent startpos endpos (current-buffer))
-    (make-overlay startpos endpos (current-buffer))))
+  (vm-make-extent startpos endpos))
 
 
 (defun vmpc-create-sig-and-pre-sig-exerlays ()
@@ -604,7 +620,8 @@ return the contents of all header fields which match that regexp,
 separated from each other by CLUMP-SEP."
   (if (and (eq vmpc-current-buffer 'none)
 	   (memq vmpc-current-state '(reply forward resend)))
-      (let ((mp (car (vm-select-marked-or-prefixed-messages 1)))
+      (let ((mp (car (vm-select-operable-messages
+		      1 (vm-interactive-p) "Operate on")))
             content c)
         (if (not (listp hdrfield))
            (setq hdrfield (list hdrfield)))
@@ -627,7 +644,8 @@ separated from each other by CLUMP-SEP."
   (if (and (eq vmpc-current-buffer 'none)
 	   (memq vmpc-current-state '(reply forward resend)))
       (save-excursion
-	(let* ((mp (car (vm-select-marked-or-prefixed-messages 1)))
+	(let* ((mp (car (vm-select-operable-messages
+			 1 (vm-interactive-p) "Operate on")))
 	       (message (vm-real-message-of mp))
 	       start end)
 	  (set-buffer (vm-buffer-of message))
@@ -1005,7 +1023,8 @@ field `vmpc-profile' to the records which is a sexp not meant to be edited."
     (when vmpc-auto-profiles-expunge-days
       (setq vmpc-auto-profiles
             (mapcar (lambda (p)
-                      (if (> (- today (cddr p)) vmpc-auto-profiles-expunge-days)
+                      (if (> (- today (cddr p)) 
+			     vmpc-auto-profiles-expunge-days)
                           nil
                         p))
                     vmpc-auto-profiles))
@@ -1022,7 +1041,8 @@ If no email address in found in STR, returns nil."
       (match-string 0 str)))
 
 (defun vmpc-split (string separators)
-  "Return a list by splitting STRING at SEPARATORS and trimming all whitespace."
+  "Return a list by splitting STRING at SEPARATORS and trimming all
+whitespace." 
   (let (result
         (not-separators (concat "^" separators)))
     (save-excursion
@@ -1044,7 +1064,7 @@ If no email address in found in STR, returns nil."
       (erase-buffer))
     (nreverse result)))
 
-(defun vmpc-read-actions (&optional prompt default)
+(defun vmpc-read-actions (prompt &optional default)
   "Read a list of actions to run and store it in `vmpc-actions-to-run'.
 The special action \"none\" will result in an empty action list."
   (interactive (list "VMPC actions%s: "))
@@ -1059,7 +1079,7 @@ The special action \"none\" will result in an empty action list."
           (setq actions default)
         (setq actions (vmpc-split actions " "))
         (setq actions (reverse actions))))
-    (when (interactive-p)
+    (when (vm-interactive-p)
       (setq vmpc-actions-to-run actions)
       (message "VMPC actions to run: %S" actions))
     actions))
@@ -1121,11 +1141,12 @@ PROMPT argument and call this function interactively in the composition buffer."
   (if (or (and (eq vmpc-current-buffer 'none)
 	       (not (eq vmpc-current-state 'automorph)))
 	  (eq vmpc-current-state 'automorph))
-      (let ((headers (or (assoc vmpc-current-buffer vmpc-prompt-for-profile-headers)
-                         (assoc vmpc-current-state vmpc-prompt-for-profile-headers)
-                         (assoc 'default vmpc-prompt-for-profile-headers)))
+      (let ((headers 
+	     (or (assoc vmpc-current-buffer vmpc-prompt-for-profile-headers)
+		 (assoc vmpc-current-state vmpc-prompt-for-profile-headers)
+		 (assoc 'default vmpc-prompt-for-profile-headers)))
             addrs a old-actions actions dest)
-        (setq headers (car (cdr headers)))
+        (setq headers (cadr headers))
         ;; search also other headers for known addresses 
         (while (and headers (not actions))
           (setq addrs (vmpc-get-header-contents (car headers)))
@@ -1139,7 +1160,8 @@ PROMPT argument and call this function interactively in the composition buffer."
             (setq addrs (cdr addrs)))
           (setq headers (cdr headers)))
 
-        (setq dest (or dest vmpc-default-profile (if prompt (vmpc-read-profile))))
+        (setq dest 
+	      (or dest vmpc-default-profile (if prompt (vmpc-read-profile))))
         
         (unless actions 
           (setq actions (vmpc-get-profile-for-address dest)))
@@ -1159,27 +1181,33 @@ PROMPT argument and call this function interactively in the composition buffer."
             (setq remember t)
             (setq actions (list actions)))
 
-          ;; save the association of this profile with these actions if applicable
+          ;; save the association of this profile with these actions
+	  ;; if applicable 
           (if (and (not (equal old-actions actions))
                    (or (eq remember t)
                        (and (eq remember 'prompt)
                             (if actions 
-                                (y-or-n-p (format "Always run %s for \"%s\"? "
-                                                  actions dest))
+                                (y-or-n-p 
+				 (format "Always run %s for \"%s\"? "
+					 actions dest))
                               (if (vmpc-get-profile-for-address dest)
-                                  (yes-or-no-p (format "Delete profile for \"%s\"? "
-                                                       dest)))))))
+                                  (yes-or-no-p 
+				   (format "Delete profile for \"%s\"? "
+					   dest)))))))
               (vmpc-save-profile-for-address dest actions))
           
           ;; TODO: understand when vmpc-prompt-for-profile has to run actions 
-          ;; if we are in automorph (actually being called from within an action)
+          ;; if we are in automorph (actually being called from within
+          ;; an action) 
           (if (eq vmpc-current-state 'automorph)
               (let ((vmpc-actions-to-run actions))
                 (vmpc-run-actions))
-            ;; otherwise add the actions to the end of the list as a side effect 
+            ;; otherwise add the actions to the end of the list as a
+	    ;; side effect  
             (setq vmpc-actions-to-run (append vmpc-actions-to-run actions)))
 	
-          ;; return the actions, which makes the condition true if a profile exists 
+          ;; return the actions, which makes the condition true if a
+	  ;; profile exists  
           actions))))
 
 ;; -------------------------------------------------------------------
@@ -1224,6 +1252,17 @@ haven't yet been checked when this one is checked."
   "Return true if the current folder name matches REGEXP."
   (string-match regexp (buffer-name)))
 
+(defun vmpc-folder-account-match (account-regexp)
+  "Return true if the current folder's POP/IMAP account name matches REGEXP."
+  (let ((account
+	 (cond ((eq vm-folder-access-method 'imap)
+		(vm-imap-account-name-for-spec (vm-folder-imap-maildrop-spec)))
+	       ((eq vm-folder-access-method 'pop)
+		(vm-pop-find-name-for-spec (vm-folder-pop-maildrop-spec)))
+	       (t "")
+	       )))
+    (string-match account-regexp account)))
+
 (defun vmpc-header-match (hdrfield regexp &optional clump-sep num)
   "Return true if the contents of specified header HDRFIELD match REGEXP.
 For automorph, this means the header in your message, when replying it means
@@ -1244,14 +1283,16 @@ If NUM is specified return the match string NUM."
                 (if num (match-string num hdr) t))))))
 
 (defun vmpc-only-from-match (hdrfield regexp &optional clump-sep)
-  "Return non-nil if all emails from the given HDRFIELD are matched by REGEXP."
+  "Return non-nil if all emails from the given HDRFIELD are matched by
+REGEXP." 
   (let* ((content (vmpc-get-header-contents hdrfield clump-sep))
          (case-fold-search t)
          (pos 0)
          (len (length content))
          (only-from (not (null content))))
     (while (and only-from (< pos len)
-                (setq pos (string-match "[a-z0-9._-]+@[a-z0-9._-]+" content pos)))
+                (setq pos (string-match "[a-z0-9._-]+@[a-z0-9._-]+" 
+					content pos)))
       (if (not (string-match regexp (match-string 0 content)))
           (setq only-from nil))
       (setq pos (1+ pos)))
@@ -1259,8 +1300,8 @@ If NUM is specified return the match string NUM."
 
 (defun vmpc-body-match (regexp)
   "Return non-nil if the contents of the message body match REGEXP.
-For automorph, this means the body of your message; when replying it means the
-body of the message being replied to."
+For automorph, this means the body of your message; when replying it
+means the body of the message being replied to."
   (cond ((and (memq vmpc-current-state '(reply forward resend))
 	      (eq vmpc-current-buffer 'none))
 	 (string-match regexp (vmpc-get-replied-body-text)))
@@ -1293,29 +1334,31 @@ Run this function in order to test/check your conditions."
                                         nil t nil nil "reply"))
             vmpc-current-buffer 'none))
     (vm-follow-summary-cursor)
-    (vm-select-folder-buffer)
-    (vm-check-for-killed-summary)
-    (vm-error-if-folder-empty)
+    (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
     (vmpc-build-true-conditions-list)
     (message "VMPC true conditions: %S" vmpc-true-conditions)
     vmpc-true-conditions))
 
 (defun vmpc-build-true-conditions-list ()
-  "Built list of true conditions and store it in variable `vmpc-true-conditions'."
+  "Build list of true conditions and store it in the variable 
+`vmpc-true-conditions'."
+  (interactive)
   (setq vmpc-true-conditions nil)
-  (mapcar (lambda (c)
-            (if (save-excursion (eval (cons 'progn (cdr c))))
-                (setq vmpc-true-conditions (cons (car c) vmpc-true-conditions))))
-          vmpc-conditions)
+  (mapc
+   (lambda (c)
+     (if (save-excursion (eval (cons 'progn (cdr c))))
+	 (setq vmpc-true-conditions (cons (car c) vmpc-true-conditions))))
+   vmpc-conditions)
   (setq vmpc-true-conditions (reverse vmpc-true-conditions)))
 
 (defun vmpc-build-actions-to-run-list ()
-  "Built a list of the actions to run.
+  "Build a list of the actions to run.
 These are the true conditions mapped to actions.  Duplicates will be
 eliminated.  You may run it in a composition buffer in order to see what
 actions will be run."
   (interactive)
-  (if (and (interactive-p) (not (member major-mode '(vm-mail-mode mail-mode))))
+  (if (and (vm-interactive-p) 
+	   (not (member major-mode '(vm-mail-mode mail-mode))))
       (error "Run `vmpc-build-actions-to-run-list' in a composition buffer!"))
   (let ((alist (or (symbol-value (intern (format "vmpc-%s-alist"
                                                  vmpc-current-state)))
@@ -1323,17 +1366,20 @@ actions will be run."
         (old-vmpc-actions-to-run vmpc-actions-to-run)
         actions)
     (setq vmpc-actions-to-run nil)
-    (mapcar (lambda (c)
-              (setq actions (cdr (assoc c alist)))
-              ;; TODO: warn about unbound conditions?
-              (while actions
-                (if (not (member (car actions) vmpc-actions-to-run))
-                    (setq vmpc-actions-to-run (cons (car actions) vmpc-actions-to-run)))
-                (setq actions (cdr actions))))
-            vmpc-true-conditions)
+    (mapc
+     (lambda (c)
+       (setq actions (cdr (assoc c alist)))
+       ;; TODO: warn about unbound conditions?
+       (while actions
+	 (if (not (member (car actions) vmpc-actions-to-run))
+	     (setq vmpc-actions-to-run 
+		   (cons (car actions) vmpc-actions-to-run)))
+	 (setq actions (cdr actions))))
+     vmpc-true-conditions)
     (setq vmpc-actions-to-run (reverse vmpc-actions-to-run))
-    (setq vmpc-actions-to-run (append vmpc-actions-to-run old-vmpc-actions-to-run)))
-  (if (interactive-p)
+    (setq vmpc-actions-to-run 
+	  (append vmpc-actions-to-run old-vmpc-actions-to-run)))
+  (if (vm-interactive-p)
       (message "VMPC actions to run: %S" vmpc-actions-to-run))
   vmpc-actions-to-run)
 
@@ -1344,17 +1390,17 @@ actions will be run."
 If called interactivly it promts for the regexp.  You may also use
 completion."
   (interactive)
-  (let ((action-names (mapcar '(lambda (a)
-                                 (list (regexp-quote (car a)) 1))
+  (let ((action-names (mapcar (lambda (a)
+				(list (regexp-quote (car a)) 1))
                               vmpc-actions)))
     (if (not action-regexp)
         (setq action-regexp (completing-read "VMPC action-regexp: "
                                              action-names)))
-    (mapcar '(lambda (action)
-               (if (string-match action-regexp (car action))
-                   (mapcar '(lambda (action-command)
-                              (eval action-command))
-                           (cdr action))))
+    (mapcar (lambda (action)
+	      (if (string-match action-regexp (car action))
+		  (mapcar (lambda (action-command)
+			    (eval action-command))
+			  (cdr action))))
             vmpc-actions)))
 
 
@@ -1364,8 +1410,8 @@ If verbose is supplied, it should be a STRING, indicating the name of a
 buffer to which to write diagnostic output."
   (interactive)
   
-  (if (and (not vmpc-actions-to-run) (not actions) (interactive-p))
-      (setq vmpc-actions-to-run (vmpc-read-actions)))
+  (if (and (not vmpc-actions-to-run) (not actions) (vm-interactive-p))
+      (setq vmpc-actions-to-run (vmpc-read-actions "Actions: ")))
 
   (let ((actions (or actions vmpc-actions-to-run)) form)
     (while actions
@@ -1467,9 +1513,22 @@ recursion nor concurrent calls."
   "*Forward a message with pcrisis voodoo."
   ;; this stuff is already done when replying, but not here:
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
-  (vm-error-if-folder-empty)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
+  ;;  the rest is almost exactly the same as replying:
+  (vmpc-init-vars 'forward)
+  (vmpc-build-true-conditions-list)
+  (vmpc-build-actions-to-run-list)
+  (vmpc-run-actions)
+  ad-do-it
+  (vmpc-create-sig-and-pre-sig-exerlays)
+  (vmpc-make-vars-local)
+  (vmpc-run-actions))
+
+(defadvice vm-forward-message-plain (around vmpc-forward activate)
+  "*Forward a message in plain text with pcrisis voodoo."
+  ;; this stuff is already done when replying, but not here:
+  (vm-follow-summary-cursor)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   ;;  the rest is almost exactly the same as replying:
   (vmpc-init-vars 'forward)
   (vmpc-build-true-conditions-list)
@@ -1484,9 +1543,7 @@ recursion nor concurrent calls."
   "*Resent a message with pcrisis voodoo."
   ;; this stuff is already done when replying, but not here:
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
-  (vm-error-if-folder-empty)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   ;; the rest is almost exactly the same as replying:
   (vmpc-init-vars 'resend)
   (vmpc-build-true-conditions-list)
@@ -1528,6 +1585,4 @@ Call `vmpc-no-automorph' to disable it for the current buffer."
     (vmpc-build-actions-to-run-list)
     (vmpc-run-actions)))
 
-(provide 'vm-pcrisis)
-
 ;;; vm-pcrisis.el ends here
