@@ -1,5 +1,7 @@
 ;;; vm-undo.el --- Commands to undo message attribute changes in VM
 ;;
+;; This file is part of VM
+;;
 ;; Copyright (C) 1989-1995 Kyle E. Jones
 ;; Copyright (C) 2003-2006 Robert Widhopf-Fenk
 ;;
@@ -18,14 +20,26 @@
 ;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 ;;; Code:
-(defun vm-set-buffer-modified-p (flag &optional buffer)
-  (save-excursion
-    (and buffer (set-buffer buffer))
-    (set-buffer-modified-p flag)
-    (vm-increment vm-modification-counter)
-    (intern (buffer-name) vm-buffers-needing-display-update)
-    (if (null flag)
-	(setq vm-messages-not-on-disk 0))))
+
+(provide 'vm-undo)
+
+(eval-when-compile
+  (require 'vm-misc)
+  (require 'vm-menu)
+  (require 'vm-minibuf)
+  (require 'vm-folder)
+  (require 'vm-summary)
+  (require 'vm-window)
+  (require 'vm-page)
+  (require 'vm-motion)
+  )
+
+;; vm-undo-record-list is a buffer-local-variable containing
+;; undo-records.
+;; An undo-record has:
+;; - action
+;; - message
+;; - args
 
 (defun vm-undo-boundary ()
   (if (car vm-undo-record-list)
@@ -36,10 +50,9 @@
     (mapatoms (function
 	       (lambda (b)
 		 (setq b (get-buffer (symbol-name b)))
-		 (if b
-		     (progn
-		       (set-buffer b)
-		       (vm-undo-boundary)))))
+		 (when b
+		   (set-buffer b)
+		   (vm-undo-boundary))))
 	      vm-buffers-needing-undo-boundaries)
     (fillarray vm-buffers-needing-undo-boundaries 0)))
 
@@ -96,17 +109,17 @@
     (if (equal '(nil) vm-undo-record-list)
 	(setq vm-undo-record-list nil)))
   ;; for the Undo button on the menubar, if present
-  (and (null vm-undo-record-list)
-       (vm-menu-support-possible-p)
-       (vm-menu-xemacs-menus-p)
-       (vm-menu-set-menubar-dirty-flag)))
+  (when (and (null vm-undo-record-list)
+	     (vm-menu-support-possible-p)
+	     (vm-menu-xemacs-menus-p))
+    (vm-menu-set-menubar-dirty-flag)))
 	    
 (defun vm-undo-record (sexp)
   ;; for the Undo button on the menubar, if present
-  (and (null vm-undo-record-list)
-       (vm-menu-support-possible-p)
-       (vm-menu-xemacs-menus-p)
-       (vm-menu-set-menubar-dirty-flag))
+  (when (and (null vm-undo-record-list)
+	     (vm-menu-support-possible-p)
+	     (vm-menu-xemacs-menus-p))
+    (vm-menu-set-menubar-dirty-flag))
   (setq vm-undo-record-list (cons sexp vm-undo-record-list)))
 
 (defun vm-undo-describe (record)
@@ -123,7 +136,7 @@
 	(m (nth 1 record))
 	labels)
     (cond (cell
-	   (message "VM Undo! %s/%s %s -> %s"
+	   (vm-inform 1 "VM Undo! %s/%s %s -> %s"
 		    (buffer-name (vm-buffer-of m))
 		    (vm-number-of m)
 		    (if (nth 2 record)
@@ -134,7 +147,7 @@
 		      (nth 2 cell))))
 	  ((eq (car cell) 'vm-set-labels)
 	   (setq labels (nth 2 record))
-	   (message "VM Undo! %s/%s %s%s"
+	   (vm-inform 1 "VM Undo! %s/%s %s%s"
 		    (buffer-name (vm-buffer-of m))
 		    (vm-number-of m)
 		    (if (null labels)
@@ -155,7 +168,7 @@
 	;; make folder read-only to avoid modifications when we
 	;; do this.
 	(let ((vm-folder-read-only t))
-	  (vm-preview-current-message)))))
+	  (vm-present-current-message)))))
 
 ;;;###autoload
 (defun vm-undo ()
@@ -164,15 +177,14 @@ Consecutive invocations of this command cause sequentially earlier
 changes to be undone.  After an intervening command between undos,
 the undos themselves become undoable."
   (interactive)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 0 (vm-interactive-p))
   (vm-error-if-folder-read-only)
   (vm-display nil nil '(vm-undo) '(vm-undo))
   (let ((modified (buffer-modified-p)))
-    (if (not (eq last-command 'vm-undo))
-	(setq vm-undo-record-pointer vm-undo-record-list))
-    (if (not vm-undo-record-pointer)
-	(error "No further VM undo information available"))
+    (unless (eq last-command 'vm-undo)
+      (setq vm-undo-record-pointer vm-undo-record-list))
+    (unless vm-undo-record-pointer
+      (error "No further VM undo information available"))
     ;; skip current record boundary
     (setq vm-undo-record-pointer (cdr vm-undo-record-pointer))
     (while (car vm-undo-record-pointer)
@@ -180,8 +192,8 @@ the undos themselves become undoable."
       (vm-undo-describe (car vm-undo-record-pointer))
       (eval (car vm-undo-record-pointer))
       (setq vm-undo-record-pointer (cdr vm-undo-record-pointer)))
-    (and modified (not (buffer-modified-p))
-	 (delete-auto-save-file-if-necessary))
+    (when (and modified (not (buffer-modified-p)))
+      (delete-auto-save-file-if-necessary))
     (vm-update-summary-and-mode-line)))
 
 ;;;###autoload
@@ -207,14 +219,13 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
       (vm-read-string "Set attributes: " vm-supported-attribute-names t)
       (prefix-numeric-value current-prefix-arg))))
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-error-if-folder-read-only)
-  (vm-error-if-folder-empty)
   (vm-display nil nil '(vm-set-message-attributes)
 	      '(vm-set-message-attributes))
   (let ((name-list (vm-parse string "[ \t]*\\([^ \t]+\\)"))
-	(m-list (vm-select-marked-or-prefixed-messages count))
+	(m-list (vm-select-operable-messages 
+		 count (vm-interactive-p) "Set attributes of"))
 	n-list name m)
     (while m-list
       (setq m (car m-list)
@@ -290,18 +301,24 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 	 (completion-ignore-case t))
      ;; so the user can see what message they are about to
      ;; modify.
-     (vm-follow-summary-cursor)
-     (vm-select-folder-buffer)
-     (list
-      (vm-read-string "Add labels: "
-		      (vm-obarray-to-string-list vm-label-obarray) t)
-      (prefix-numeric-value current-prefix-arg))))
-  (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
-  (vm-error-if-folder-read-only)
-  (vm-error-if-folder-empty)
-  (vm-add-or-delete-message-labels string count 'all))
+     (save-current-buffer
+       (vm-follow-summary-cursor)
+       (vm-select-folder-buffer)
+       (list
+	(vm-read-string "Add labels: "
+			(vm-obarray-to-string-list vm-label-obarray) t)
+	(prefix-numeric-value current-prefix-arg)))))
+  (let ((m-list nil)
+	(ignored-labels nil))
+    (vm-follow-summary-cursor)
+    (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
+    (vm-error-if-folder-read-only)
+    (setq m-list (vm-select-operable-messages
+		  count (vm-interactive-p) "Add labels to"))
+    (setq ignored-labels 
+	  (vm-add-or-delete-message-labels string m-list 'all))
+    (if ignored-labels
+	(vm-inform 1 "Label %s could not be added" string))))
 
 ;;;###autoload
 (defun vm-add-existing-message-labels (string count)
@@ -330,19 +347,20 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 	 (completion-ignore-case t))
      ;; so the user can see what message they are about to
      ;; modify.
+     (save-current-buffer
      (vm-follow-summary-cursor)
      (vm-select-folder-buffer)
      (list
       (vm-read-string "Add labels: "
 		      (vm-obarray-to-string-list vm-label-obarray) t)
-      (prefix-numeric-value current-prefix-arg))))
+      (prefix-numeric-value current-prefix-arg)))))
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-error-if-folder-read-only)
-  (vm-error-if-folder-empty)
-  (let ((ignored-labels
-	 (vm-add-or-delete-message-labels string count 'existing-only)))
+  (let* ((m-list (vm-select-operable-messages
+		  count (vm-interactive-p) "Add labels to"))
+	(ignored-labels
+	 (vm-add-or-delete-message-labels string m-list 'existing-only)))
     (if ignored-labels
 	(progn
 	  (set-buffer (get-buffer-create "*Ignored Labels*"))
@@ -375,29 +393,37 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 	 (completion-ignore-case t))
      ;; so the user can see what message they are about to
      ;; modify.
+     (save-current-buffer
      (vm-follow-summary-cursor)
      (vm-select-folder-buffer)
      (list
       (vm-read-string "Delete labels: "
 		      (vm-obarray-to-string-list vm-label-obarray) t)
-      (prefix-numeric-value current-prefix-arg))))
+      (prefix-numeric-value current-prefix-arg)))))
   (vm-follow-summary-cursor)
-  (vm-select-folder-buffer)
-  (vm-check-for-killed-summary)
+  (vm-select-folder-buffer-and-validate 1 (vm-interactive-p))
   (vm-error-if-folder-read-only)
-  (vm-error-if-folder-empty)
-  (vm-add-or-delete-message-labels string count nil))
+  (let ((m-list (vm-select-operable-messages
+		 count (vm-interactive-p) "Delete labels to")))
+    (vm-add-or-delete-message-labels string m-list nil)))
 
-(defun vm-add-or-delete-message-labels (string count add)
+(defun vm-add-or-delete-message-labels (string m-list add)
+  "Add or delete the labels given in STRING for all messages in
+M-LIST.  The third parameter ADD is one of:
+
+nil	       delete the label
+'all           add the label in all cases
+'existing-only add the label only if it is already existing in the folder
+							USR, 2010-12-20
+"
   (vm-display nil nil '(vm-add-message-labels vm-delete-message-labels)
 	      (list this-command))
   (setq string (downcase string))
-  (let ((m-list (vm-select-marked-or-prefixed-messages count))
-	(action-labels (vm-parse string
+  (let ((action-labels (vm-parse string
 "[\000-\040,\177-\377]*\\([^\000-\040,\177-\377]+\\)[\000-\040,\177-\377]*"))
 	(ignored-labels nil)
 	labels act-labels m mm-list)
-    (if (and add m-list)
+    (when (and add m-list)
 	(if (eq add 'all)
 	    (progn
 	      (setq act-labels action-labels)
@@ -412,27 +438,24 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 		(setq ignored-labels (cons (car act-labels) ignored-labels)))
 	      (setq act-labels (cdr act-labels)))
 	    (setq action-labels newlist))))
-    (if (null action-labels)
+    (unless action-labels
 	(setq m-list nil))
     (while m-list
       (setq m (car m-list))
-      (if (and add (vm-virtual-message-p m))
+      (when (and add (vm-virtual-message-p m))
+	(let ((labels action-labels))
+	  (with-current-buffer (vm-buffer-of (vm-real-message-of m))
+	    (while labels
+	      (intern (car labels) vm-label-obarray)
+	      (setq labels (cdr labels))))))
+      (when add
+	(dolist (mm (vm-virtual-messages-of m))
 	  (let ((labels action-labels))
-	    (save-excursion
-	      (set-buffer (vm-buffer-of (vm-real-message-of m)))
-	      (while labels
-		(intern (car labels) vm-label-obarray)
-		(setq labels (cdr labels))))))
-      (if add
-	  (save-excursion
-	    (setq mm-list (vm-virtual-messages-of m))
-	    (while mm-list
-	      (let ((labels action-labels))
-		(set-buffer (vm-buffer-of (car mm-list)))
+	    (when (buffer-name (vm-buffer-of mm))
+	      (with-current-buffer (vm-buffer-of mm)
 		(while labels
 		  (intern (car labels) vm-label-obarray)
-		  (setq labels (cdr labels))))
-	      (setq mm-list (cdr mm-list)))))
+		  (setq labels (cdr labels))))))))
       (setq act-labels action-labels
 	    labels (copy-sequence (vm-labels-of (car m-list))))
       (if add
@@ -442,8 +465,8 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 	(while act-labels
 	  (setq labels (vm-delqual (car act-labels) labels)
 		act-labels (cdr act-labels))))
-      (if add
-	  (setq labels (vm-delete-duplicates labels)))
+      (when add
+	(setq labels (vm-delete-duplicates labels)))
       (vm-set-labels (car m-list) labels)
       (vm-set-attribute-modflag-of (car m-list) t) ; added by USR
       (setq m-list (cdr m-list)))
@@ -451,50 +474,103 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
     ignored-labels))
 
 (defun vm-set-xxxx-flag (m flag norecord function attr-index)
+  "A generic function to set the message flag of M at ATTR-INDEX to
+  the value FLAG.  The argument FUNCTION tells the specific
+  non-generic function that invoked this one.  A boolean flag is
+  returned indicating success or failure of the operation.
+The flag is also set for all the virtual messages mirroring M as well
+  as the real message underlying M. 
+Normally, a record of the change is kept for the purpose of undo, and
+  the changed attributes are stuffed into the folder, but NORECORD
+  suppresses all of this.                             USR 2010-04-06" 
   (let ((m-list nil) vmp)
-    (cond
-     ((and (not vm-folder-read-only)
-	   (or (not (vm-virtual-messages-of m))
-	       (not (save-excursion
-		      (set-buffer
-		       (vm-buffer-of
-			 (vm-real-message-of m)))
-		      vm-folder-read-only)))
-           ;; do nothing it is is already set 
-           (not (eq flag (aref (vm-attributes-of m) attr-index))))
-      (cond
-       ((not norecord)
-	(setq vmp (cons (vm-real-message-of m) (vm-virtual-messages-of m)))
-	(while vmp
-	  (if (eq (vm-attributes-of m) (vm-attributes-of (car vmp)))
-	      (setq m-list (cons (car vmp) m-list)))
-	  (setq vmp (cdr vmp)))
+    (when (and (not vm-folder-read-only)
+	       (or (not (vm-virtual-messages-of m))
+		   (not (with-current-buffer
+			    (vm-buffer-of
+			     (vm-real-message-of m))
+			  vm-folder-read-only)))
+	       ;; do nothing it is is already set 
+	       (not (eq flag (aref (vm-attributes-of m) attr-index))))
+      (unless norecord
+	(dolist (v-m (cons (vm-real-message-of m) (vm-virtual-messages-of m)))
+	  (if (eq (vm-attributes-of m) (vm-attributes-of v-m))
+	      (setq m-list (cons v-m m-list))))
 	(if (null m-list)
 	    (setq m-list (cons m m-list)))
-	(while m-list
-	  (save-excursion
-	    (set-buffer (vm-buffer-of (car m-list)))
-	    (cond ((not (buffer-modified-p))
-		   (vm-set-buffer-modified-p t)
-		   (vm-undo-record (list 'vm-set-buffer-modified-p nil))))
-	    (vm-undo-record (list function (car m-list) (not flag)))
-;;;	    (vm-undo-boundary)
-	    (vm-increment vm-modification-counter))
-	  (setq m-list (cdr m-list)))))
+	(save-excursion
+	  (dolist (mm m-list)
+	    (when (buffer-name (vm-buffer-of mm))
+	      (set-buffer (vm-buffer-of mm))
+	      (cond ((not (buffer-modified-p))
+		     (vm-mark-folder-modified-p (vm-buffer-of mm))
+		     (vm-undo-record (list 'vm-set-buffer-modified-p nil))))
+	      (vm-undo-record (list function mm (not flag)))
+	      ;; (vm-undo-boundary)
+	      (vm-increment vm-modification-counter)))))
       (aset (vm-attributes-of m) attr-index flag)
       (vm-mark-for-summary-update m)
-      (if (not norecord)
-	  (progn
+      (unless norecord
 	    (vm-set-attribute-modflag-of m t)
 	    (if (eq vm-flush-interval t)
-		(vm-stuff-virtual-attributes m)
-	      (vm-set-stuff-flag-of m t))))))))
+		(vm-stuff-virtual-message-data m)
+	      (vm-set-stuff-flag-of m t)))
+      ;; return success result
+      t)))
+
+(defun vm-set-xxxx-cached-data-flag (m flag norecord function attr-index)
+  "A generic function to set the cached-data flag of M at ATTR-INDEX to
+  the value FLAG.  The argument FUNCTION tells the specific
+  non-generic function that invoked this one.
+The flag is also set for all the virtual messages mirroring M as well
+  as the real message underlying M. 
+Normally, a record of the change is kept for the purpose of undo, and
+  the changed attributes are stuffed into the folder, but NORECORD
+  suppresses all of this.                             USR 2010-04-06" 
+  (let ((m-list nil) vmp)
+    (when
+     (and (not vm-folder-read-only)
+	   (or (not (vm-virtual-messages-of m))
+	       (not (with-current-buffer
+		       (vm-buffer-of
+			 (vm-real-message-of m))
+		      vm-folder-read-only)))
+           ;; do nothing it is is already set 
+           (not (eq flag (aref (vm-cached-data-of m) attr-index))))
+     (unless norecord
+	(dolist (v-m (cons (vm-real-message-of m) (vm-virtual-messages-of m)))
+	  (if (eq (vm-cached-data-of m) (vm-cached-data-of v-m))
+	      (setq m-list (cons v-m m-list))))
+	(if (null m-list)
+	    (setq m-list (cons m m-list)))
+	(save-excursion
+	  (dolist (mm m-list)
+	    (when (buffer-name (vm-buffer-of mm))
+	      (set-buffer (vm-buffer-of mm))
+	      (cond ((not (buffer-modified-p))
+		     (vm-mark-folder-modified-p (vm-buffer-of mm))
+		     (vm-undo-record (list 'vm-set-buffer-modified-p nil))))
+	      (vm-undo-record (list function mm (not flag)))
+	      ;; (vm-undo-boundary)
+	      (vm-increment vm-modification-counter)))))
+      (aset (vm-cached-data-of m) attr-index flag)
+      (vm-mark-for-summary-update m)
+      (unless norecord
+	    (vm-set-attribute-modflag-of m t)
+	    (if (eq vm-flush-interval t)
+		(vm-stuff-virtual-message-data m)
+	      (vm-set-stuff-flag-of m t))))))
 
 
 (defun vm-set-labels (m labels)
+  "Set the message labels of M to the value LABELS (a list of
+  strings). 
+The labels are also set for all the virtual messages mirroring M as
+  well as the real message underlying M. 
+A record of the change is kept for the purpose of undo, and the
+  changed attributes are stuffed into the folder.        USR 2010-04-06" 
   (let ((m-list nil)
-	(old-labels (vm-labels-of m))
-	vmp)
+	(old-labels (vm-labels-of m)))
     (cond
      ((and (not vm-folder-read-only)
 	   (or (not (vm-virtual-messages-of m))
@@ -503,31 +579,41 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 		       (vm-buffer-of
 			 (vm-real-message-of m)))
 		      vm-folder-read-only))))
-      (setq vmp (cons (vm-real-message-of m) (vm-virtual-messages-of m)))
-      (while vmp
-	(if (eq (vm-attributes-of m) (vm-attributes-of (car vmp)))
-	    (setq m-list (cons (car vmp) m-list)))
-	(setq vmp (cdr vmp)))
+      (dolist (v-m (cons (vm-real-message-of m) (vm-virtual-messages-of m)))
+	(if (eq (vm-attributes-of m) (vm-attributes-of v-m))
+	    (setq m-list (cons v-m m-list))))
       (if (null m-list)
 	  (setq m-list (cons m m-list)))
-      (while m-list
-	(save-excursion
-	  (set-buffer (vm-buffer-of (car m-list)))
-	  (cond ((not (buffer-modified-p))
-		 (vm-set-buffer-modified-p t)
-		 (vm-undo-record (list 'vm-set-buffer-modified-p nil))))
-	  (vm-undo-record (list 'vm-set-labels m old-labels))
-;;;	  (vm-undo-boundary)
-	  (vm-increment vm-modification-counter))
-	(setq m-list (cdr m-list)))
+      (save-excursion
+	(dolist (mm m-list)
+	  (when (buffer-name (vm-buffer-of mm))
+	    (set-buffer (vm-buffer-of mm))
+	    (cond ((not (buffer-modified-p))
+		   (vm-mark-folder-modified-p (vm-buffer-of mm))
+		   (vm-undo-record (list 'vm-set-buffer-modified-p nil))))
+	    (vm-undo-record (list 'vm-set-labels m old-labels))
+	    ;; (vm-undo-boundary)
+	    (vm-increment vm-modification-counter))))
       (vm-set-labels-of m labels)
       (vm-set-label-string-of m nil)
       (vm-mark-for-summary-update m)
       (if (eq vm-flush-interval t)
-	  (vm-stuff-virtual-attributes m)
+	  (vm-stuff-virtual-message-data m)
 	(vm-set-stuff-flag-of m t))))))
 
 
+;; This flag is defunct, replaced by body-to-be-discarded.  USR, 2010-06-08
+(defun vm-set-headers-to-be-retrieved-flag (m flag &optional norecord)
+  nil)
+
+(defun vm-set-body-to-be-discarded-flag (m flag &optional norecord)
+  (vm-set-xxxx-cached-data-flag 
+   m flag norecord 'vm-set-body-to-be-discarded-flag 21))
+
+(defun vm-set-body-to-be-retrieved-flag (m flag &optional norecord)
+  (vm-set-xxxx-cached-data-flag 
+   m flag norecord 'vm-set-body-to-be-retrieved-flag 22))
+
 (defun vm-set-new-flag (m flag &optional norecord)
   (vm-set-xxxx-flag m flag norecord 'vm-set-new-flag 0))
 
@@ -552,6 +638,27 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 (defun vm-set-redistributed-flag (m flag &optional norecord)
   (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 8))
 
+(defun vm-set-flagged-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 9))
+
+(defun vm-set-folded-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 10))
+
+(defun vm-set-watched-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 11))
+
+(defun vm-set-ignored-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 12))
+
+(defun vm-set-read-receipt-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 13))
+
+(defun vm-set-read-receipt-sent-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 14))
+
+(defun vm-set-attachments-flag (m flag &optional norecord)
+  (vm-set-xxxx-flag m flag norecord 'vm-set-redistributed-flag 15))
+
 ;; use these to avoid undo and summary update.
 (defun vm-set-new-flag-of (m flag) (aset (aref m 2) 0 flag))
 (defun vm-set-unread-flag-of (m flag) (aset (aref m 2) 1 flag))
@@ -561,16 +668,21 @@ COUNT-1 messages to be altered.  COUNT defaults to one."
 (defun vm-set-written-flag-of (m flag) (aset (aref m 2) 5 flag))
 (defun vm-set-forwarded-flag-of (m flag) (aset (aref m 2) 6 flag))
 (defun vm-set-redistributed-flag-of (m flag) (aset (aref m 2) 8 flag))
+(defun vm-set-flagged-flag-of (m flag) (aset (aref m 2) 9 flag))
+(defun vm-set-folded-flag-of (m flag) (aset (aref m 2) 10 flag))
+(defun vm-set-watched-flag-of (m flag) (aset (aref m 2) 11 flag))
+(defun vm-set-ignored-flag-of (m flag) (aset (aref m 2) 12 flag))
+(defun vm-set-read-receipt-flag-of (m flag) (aset (aref m 2) 13 flag))
+(defun vm-set-read-receipt-sent-flag-of (m flag) (aset (aref m 2) 14 flag))
+(defun vm-set-attachments-flag-of (m flag) (aset (aref m 2) 15 flag))
 
-;; this is solely for the use of vm-stuff-attributes and appears here
-;; only because this function should be grouped with others of its kind
-;; for maintenance purposes.
+;; this is solely for the use of vm-stuff-message-data and
+;; appears here only because this function should be grouped with
+;; others of its kind for maintenance purposes.
 (defun vm-set-deleted-flag-in-vector (v flag)
   (aset v 2 flag))
 ;; ditto.  this is for vm-read-attributes.
 (defun vm-set-new-flag-in-vector (v flag)
   (aset v 0 flag))
 
-(provide 'vm-undo)
-
 ;;; vm-undo.el ends here
